
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                       ; 
;simple activation model with the preparedness          ;
;                                                       ;
;writes out which chunk it expects in output            ;
;                                                       ;
;model used in activation_paper                         ;
;                                                       ;
;do-subj runs all trials of all participants            ;
;do-id n runs id n                                      ;
;*curr-trial* shows the last presented trial            ;
;                                                       ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;Parameter Definitions;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;Running Code Parameters;;;;;;;;;;;;;;;;;;;;;;;;;
(defparameter *curr-stim* nil)
(defparameter *curr-trial* nil)
(defparameter *cheat-trial* nil)
(defparameter *task* nil)
(defparameter *start-t-p* nil)
(defparameter *stop-t-p* nil)
(defparameter *p-rt* nil)
(defparameter *start-t-d* nil)
(defparameter *stop-t-d* nil)
(defparameter *d-rt* nil)
(defparameter *probe-resp* nil)
(defparameter *diag-resp* nil)
(defparameter *trial-list* nil)

;;;;;;;;;;;;;;;;;;;;;;;;;Running Code Support - Parameters;;;;;;;;;;;;;;;;;;;;;;;;;
;(specifies positions and stores relevant information for running code)
(defparameter *probe-pos* nil)
(defparameter *probe-corr* nil)
(defparameter *probe-done* nil);added
(defparameter *diag-corr* nil)
(defparameter *done* nil);added
(defparameter *trial-num* 0)
(defparameter *subj-nr* 0)
(defparameter *idm* 0)
(defparameter *SN* 0)

(defparameter *expect1* nil) ;new for expect
(defparameter *expect2* nil)
(defparameter *expect3* nil)
(defparameter *expect4* nil)

;;;;;;;;;;;;;;;;;;;;;;;;;Analysis Code for Accuracy Data - Parameters;;;;;;;;;;;;;;;;;;;;;;;;;
(defparameter *p-acc* (list 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0))
(defparameter *d-acc* (list 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0))
(defparameter *acc-c* (list 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0))
(defparameter *diag-acc* 0)
(defparameter *diag-count* 0)

;;;;;;;;;;;;;;;;;;;;;;;;;Analysis Code for RT Data - Parameters;;;;;;;;;;;;;;;;;;;;;;;;;
(defparameter *p-rt-i* (list 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0))
(defparameter *p-c-i* (list 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0))
(defparameter *d-rt-i* (list 0 0))
(defparameter *d-c-i* (list 0 0))

;;;;;;;;;;;;;;;;;;;;;;;;;Output Code - Parameters;;;;;;;;;;;;;;;;;;;;;;;;;
(defparameter *read-write-path* (current-pathname))
(defparameter *results* nil)
(defparameter *all-subj-indiv-rt* nil)
(defparameter *all-subj-indiv-acc-p* nil)
(defparameter *all-subj-indiv-acc-d* nil)
(defparameter *mean-rt* nil)
(defparameter *mean-p-acc* nil)
(defparameter *mean-d-acc* nil)
(defparameter *mm-rt* nil)
(defparameter *mm-p-acc* nil)
(defparameter *mm-d-acc* nil)
(defparameter *Fit-probes* 0)
(defparameter *Fit-diagnosis* 0)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;Presentation Functions;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun do-subj ()
  (read-file)
  (dotimes (i (length *trial-list*)) (do-exp nil (+ i 1)))
  (write-file-res)
 
)

(defun do-exp (&optional allow-read (subj 1));needs to be set to t if running do-exp alone
  (reset)
  (when allow-read;used when running do-exp alone

    (read-file)
    (setf *idm* 0))

  (setf *trial-num* 0)
  (setf *subj-nr* subj)
  (dolist (i (nth (- subj 1) *trial-list*))
    (do-trial (setf *curr-trial* i)))

   (when allow-read;used when running do-exp alone

    (write-file-res))
)

(defun do-set (n)
  (reset)
  (read-file)
  (setf *trial-num* 0)
  (setf *idm* 0)
  (dotimes (i n) (do-trial (setf *curr-trial* (nth i (first *trial-list*)))))
  (dolist (i (reverse *results*))
    (format t "~%~{~t~a~}" i))
)

(defun do-id (id)
  (reset)
  (read-file)
  (setf *trial-num* 0)
  (setf *idm* 0)
  (dolist (trial (first *trial-list*)) 
    (if (= (second (reverse trial)) id) (setf *curr-trial* trial)))
  (do-trial *curr-trial*)
)

(defun do-trial (trial)
  (reset)
  (analyze-trial trial)
  (incf *trial-num*)
  (incf *idm*)

  (setf *expect1* nil)
  (setf *expect2* nil)
  (setf *expect3* nil)
  (setf *expect4* nil)

  (setf *task* nil)

;event based code
;
  (setf *cheat-trial* trial)
  
  (let ((window (open-exp-window "Diagnosis Experiment" :visible nil)))
    (install-device window)

    (dotimes (i (+ 1 *probe-pos*))
      (schedule-event-relative (* i 3) 'display-next))

    (run-until-condition (lambda () *probe-done*) :real-time nil)

    (dotimes (i (- (length *curr-trial*) (+ 6 *probe-pos*)))
      (schedule-event-relative (+ 1 (* i 3)) 'display-next));the additive term determines the time lag after the probe reaction was given
    (run-until-condition (lambda () *done*) :real-time nil));altered
;

  (save-results-acc)

)

(defun display-next ()
 (clear-exp-window)
      (add-text-to-exp-window :text (setf *curr-stim* (format nil "~a" (pop *cheat-trial*))) :x 50 :y 150)
      
      (when (equalp *curr-stim* "diagnosis")
        (setf *start-t-d* (mp-time))
        (setf *task* "diag"))
      
      (when (and (= (length *curr-stim*) 1) 
                 (not (equalp *curr-stim* "diagnosis"))) 
        (setf *start-t-p* (mp-time))
        (setf *task* "probe"))
      

      (if (not (or;needed to run model with constant SA across all symptoms
                (equalp (format nil "~a" *curr-stim*) "diagnosis") 
                (= (length (format nil "~a" *curr-stim*)) 1))) 
          (incf *SN*))
      
      (proc-display)
)

(defmethod rpm-window-key-event-handler ((win rpm-window) key)
  (when (equalp *task* "probe")
    (setf *probe-resp* (string-upcase (string key)))
    (setf *stop-t-p* (mp-time))
    (setf *probe-done* t);added
    (calc-rt *start-t-p* *stop-t-p*))
  (when (equalp *task* "diag")
    (setf *diag-resp* (string-upcase (string key)))
    (setf *stop-t-d* (mp-time))
    (setf *done* t);added
    (calc-rt *start-t-d* *stop-t-d*))
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;Auxiliary Functions;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defun save-results-acc ()
  (let ((probe-acc) (diag-acc) (probe-type) (probe-id (third (reverse *curr-trial*))))
    (if (equalp *probe-corr* *probe-resp*) 
        (setf probe-acc 1) 
        (setf probe-acc 0))
    (if (equalp *diag-corr* *diag-resp*)
        (setf diag-acc 1)
        (setf diag-acc 0))

  (case (nth (- (length *curr-trial*) 3) *curr-trial*)
    (1 (setf probe-type "relevant"))
    (2 (setf probe-type "irrelevant"))
    (3 (setf probe-type "discarded_0"))
    (4 (setf probe-type "discarded_1"))
    (5 (setf probe-type "discarded_2"))
    (6 (setf probe-type "distractor")))

    (push (list *idm* *subj-nr* *trial-num* *p-rt* *probe-resp* *probe-corr* probe-acc *d-rt* *diag-resp* *diag-corr* diag-acc *probe-pos* probe-type *expect1* *expect2* *expect3* *expect4*) *results*);;altered for output of symptoms

))

(defun analyze-trial (trial)
  (setf *start-t-p* nil)
  (setf *stop-t-p* nil)
  (setf *p-rt* nil)
  (setf *start-t-d* nil)
  (setf *stop-t-d* nil)
  (setf *d-rt* nil)
  (setf *probe-resp* nil)
  (setf *diag-resp* nil)
  (setf *probe-pos* nil)
  (setf *probe-corr* nil)
  (setf *diag-corr* nil)
  (setf *probe-done* nil)
  (setf *done* nil)


  (setf *probe-pos* (fourth (reverse trial)))

  (when (not (null *probe-pos*))
    (if (null (find (nth *probe-pos* trial) '(b t w k h p q m g)))
        (setf *probe-corr* (string 'n))
        (setf *probe-corr* (string 'y))))
  (setf *diag-corr* (format nil "~a" (fifth (reverse trial))))
)


;;;;;;;;;;;;;;;;;;;;;;;;;Calculation Functions;;;;;;;;;;;;;;;;;;;;;;;;;
(defun calc-rt (start-t stop-t)
  (if (equalp *task* "probe")
      (setf *p-rt* (- stop-t start-t))
      (setf *d-rt* (- stop-t start-t))))

(defun calc-hook (init decay root)
  (let ((results) (time 3))
    (push init results)
    (dotimes (i 3) 
      (push (setf init (* init (expt (- 1 decay) (expt time root) ))) results)
      (setf time (+ 3 time)))
    (reverse results)
))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;Input Functions;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;NOTE: trials need to be formated as follows:
;(s1 s2 s3 s4 "diagnosis" diag-corr probe-pos probe-type id subj probe)
;set parentheses in .txt file (before reading in)
;replace gaps in trials file with "NULL"
;remove all "," from trials file

(defun read-file ()
  (with-open-file (in (merge-pathnames 
                       (make-pathname :name "trials" :directory '(:relative "input") :type "txt")  
                       *read-write-path*)
                      :direction :input)
    (do ((trial-lis nil (cons trial trial-lis))
         (trial (read in nil 'end) (read in nil 'end)))
        ((equal trial 'end) (setf *trial-list* (reverse trial-lis))))
)
  (restruc *trial-list*)
)

;trials have the following format after restructuring:
;(s1 s2 s3 s4 probe[between s1 and 'diagnosis] "diagnosis" diag-corr probe-pos probe-type id subj)
(defun restruc (trials)
  (dotimes (i (length trials))
    (let ((trial (nth i trials)) (rest))
      (let ((probe (first (reverse trial)))
            (probe-pos (fifth (reverse trial))))
        (setf trial (append (butlast trial (- 11 probe-pos)) 
                            (cons probe (nthcdr probe-pos (butlast trial)))))
        (setf (nth i *trial-list*) trial)
        )))
  (setf *trial-list* (find-null *trial-list*))
  (vp-lis *trial-list*)
)

(defun find-null (trials)
  (dotimes (l (length trials) trials)
    (setf (nth l trials) 
          (let ((i (nth l trials)))
            (dotimes (j (length i) i) 
              (if (equalp (nth j i) 'NULL) 
                  (setf i (append (butlast i (- (length i) j))
                                  (nthcdr (+ j 1) i))))))))
)

(defun vp-lis (trial-lis)
  (let ((newlis) (trial) (act-vp 0))
  (dotimes (i (first (last (first (last *trial-list*)))) newlis) (setf newlis (cons nil newlis)))
  (dotimes (j (length trial-lis) newlis)
    (setf trial (nth j trial-lis))
    (cond 
     ((= (first (last trial)) (+ act-vp 1)) (setf (nth act-vp newlis) (cons trial (nth act-vp newlis))))
     ((not (= (first (last trial)) (+ act-vp 1)))
      (progn 
        (setf (nth act-vp newlis) (reverse (nth act-vp newlis)))
        (incf act-vp)
        (setf (nth act-vp newlis) (cons trial (nth act-vp newlis))))) )
)
(setf (first (last newlis)) (reverse (first (last newlis))))
(setf *trial-list* newlis)
))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;Output Functions;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun write-file-res ()
  (with-open-file (out (ensure-directories-exist 
                        (merge-pathnames 
                         (make-pathname :name "results_dip_forep_lf1.4_mas2.5_180210" :directory '(:relative "output") :type "dat")  
                         *read-write-path*))
                       :direction :output :if-does-not-exist :create :if-exists :supersede)
    (setf *results* (reverse *results*))
    (push (list 'model_id 'subject_number 'trial_number 'probe_reaction_time 'model_probe_response 'correct_probe_response 'probe_accuracy 'diagnosis_reaction_time 'model_diagnosis 'correct_diagnosis 'diagnosis_accuracy 'probe_position 'probe_type 'expect1 'expect2 'expect3 'expect4) *results*)

    (dolist (line *results*)
      (format out "~{~a~t~}~%" line))
))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;ACT-R Model Code;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun new-w (buffer slot)
  (if (eq buffer 'imaginal )    
    (case slot
      (s1 .16)
      (s2 .16)
      (s3 .16)
      (s4 .16)
      (t nil))
   ))



(clear-all)

(define-model Katja_dip

(sgp 
:esc t                     ;enables subsymbolic computations
:trace-detail low :v nil  ;enables buffer trace
:act t                   ;enables activation trace
:show-focus t                ;shows focus of attention

:bll .5             ;enables base-level learning and sets decay parameter to 0.5
:ol 8                     ;nil: disables optimized learning (default: t) 4 = number of references for which bll is used, the rest uses optimized learning
:rt -3                     ;set retrieval threshold
:lf 1.4            ;decreases latency (retrievals are 10 times as fast
:GA 0                          ;overwrites the default goal-activation of 1
:IMAGINAL-ACTIVATION  1  ;default = 1        ;is overwritten by w-hook, but needs to be set to a value to spread anything
:w-hook new-w                  ;calls the function new-w defined above to set the weights of buffer slots

:MAS 2.5  ;default =3                                  ;enables spreading activation s = 3 
                                           ;(sji (association from source j to chunk i): = s - ln (fanj))
                                           ;fanj (number of chunks in which j appears)
:er t                      ;if several chunks have same activation, choice is made randomly
;:mp 1.0                    ;enables parial matching -set similarities before...
:ans 0.05                   ;enables transient noice (is computed with each retrieval request)

:DELETE-VISICON-CHUNKS t
:visual-movement-tolerance 10

:visual-attention-latency .035

;:save-buffer-trace t
)


(chunk-type association name value s1 s2 s3 s4 type)
(chunk-type symptom value)
(chunk-type previoussymptom s1 s2 s3 s4)
(chunk-type diagnose state response)

(chunk-type expectation position state) ;;;new

;;; add chunks for control states 

(add-dm (detect isa chunk))
(add-dm (attend isa chunk))
(add-dm (encode isa chunk))
(add-dm (start isa chunk))
(add-dm (retrieve isa chunk))
(add-dm (end isa chunk))
(add-dm (respond isa chunk))
(add-dm (done isa chunk))
(add-dm (retrieve-probe isa chunk))
(add-dm (respond-to-probe isa chunk))

;;; add chunks for symptoms 

(add-dm (Husten isa symptom value "Husten"))                ;is lisp case sensitive???
(add-dm (Atemnot isa symptom value "Atemnot"))

(add-dm (Hautrtung isa symptom value "Hautrtung"))
(add-dm (Hautvertzung isa symptom value "Hautvertzung"))

(add-dm (Erbrechen isa symptom value "Erbrechen"))  
(add-dm (Durchfall isa symptom value "Durchfall")) 

(add-dm (Kopfschmerzen isa symptom value "Kopfschmerzen"))  
(add-dm (Augenreizung isa symptom value "Augenreizung"))  
(add-dm (Juckreiz isa symptom value "Juckreiz"))
 
(add-dm (diagnosis isa symptom value "diagnosis"))

;;; add chunks for chemicals 
(add-dm (B isa association value "B" s1 Atemnot s2 Husten s3 Kopfschmerzen s4 Augenreizung type chemical))
(add-dm (T isa association value "T" s1 Atemnot s2 Husten s3 Kopfschmerzen s4 Juckreiz type chemical))  
(add-dm (W isa association value "W" s1 Husten s2 Augenreizung s3 Juckreiz type chemical))

(add-dm (Q isa association value "Q" s1 Hautrtung s2 Hautvertzung s3 Kopfschmerzen s4 Augenreizung type chemical))
(add-dm (M isa association value "M" s1 Hautrtung s2 Hautvertzung s3 Kopfschmerzen s4 Juckreiz type chemical))
(add-dm (G isa association value "G" s1 Hautvertzung s2 Augenreizung s3 Juckreiz type chemical))

(add-dm (K isa association value "K" s1 Erbrechen s2 Durchfall s3 Kopfschmerzen s4 Augenreizung type chemical))
(add-dm (H isa association value "H" s1 Erbrechen s2 Durchfall s3 Kopfschmerzen s4 Juckreiz type chemical))
(add-dm (P isa association value "P" s1 Durchfall s2 Augenreizung s3 Juckreiz type chemical))

(add-dm (F isa association value "F" type distractor))
(add-dm (Z isa association value "Z" type distractor))  
(add-dm (V isa association value "V" type distractor))
(add-dm (D isa association value "D" type distractor))
(add-dm (N isa association value "N" type distractor))
(add-dm (C isa association value "C" type distractor))  
(add-dm (X isa association value "X" type distractor))
(add-dm (L isa association value "L" type distractor))
(add-dm (R isa association value "R" type distractor))


;;;new chunks foe expectation
(add-dm (probe1 isa expectation position one state probe))
(add-dm (probe2 isa expectation position two state probe))
(add-dm (probe3 isa expectation position three state probe))
(add-dm (probe4 isa expectation position four state probe))

(add-dm (symptom1 isa expectation position one state symptom))
(add-dm (symptom2 isa expectation position two state symptom))
(add-dm (symptom3 isa expectation position three state symptom))


(add-dm (goal isa diagnose state start))


;set neg sjis 
#|
;Husten
(add-sji (Husten Q -.75))
(add-sji (Husten M -.75))
(add-sji (Husten G -.75))
(add-sji (Husten K -.75))
(add-sji (Husten H -.75))
(add-sji (Husten P -.75))

;Atemnot
(add-sji (Atemnot W -.75))
(add-sji (Atemnot Q -.75))
(add-sji (Atemnot M -.75))
(add-sji (Atemnot G -.75))
(add-sji (Atemnot K -.75))
(add-sji (Atemnot H -.75))
(add-sji (Atemnot P -.75))

;Hautrtung
(add-sji (Hautrtung B -.75))
(add-sji (Hautrtung T -.75))
(add-sji (Hautrtung W -.75))
(add-sji (Hautrtung G -.75))
(add-sji (Hautrtung K -.75))
(add-sji (Hautrtung H -.75))
(add-sji (Hautrtung P -.75))

;Hautvertzung
(add-sji (Hautvertzung B -.75))
(add-sji (Hautvertzung T -.75))
(add-sji (Hautvertzung W -.75))
(add-sji (Hautvertzung K -.75))
(add-sji (Hautvertzung H -.75))
(add-sji (Hautvertzung P -.75))

;Erbrechen
(add-sji (Erbrechen B -.75))
(add-sji (Erbrechen T -.75))
(add-sji (Erbrechen W -.75))
(add-sji (Erbrechen Q -.75))
(add-sji (Erbrechen M -.75))
(add-sji (Erbrechen G -.75))
(add-sji (Erbrechen P -.75))

;Durchfall
(add-sji (Durchfall B -.75))
(add-sji (Durchfall T -.75))
(add-sji (Durchfall W -.75))
(add-sji (Durchfall Q -.75))
(add-sji (Durchfall M -.75))
(add-sji (Durchfall G -.75))

;Kopfschmerzen
(add-sji (Kopfschmerzen W -.75))
(add-sji (Kopfschmerzen G -.75))
(add-sji (Kopfschmerzen P -.75))

;Augenreizung
(add-sji (Augenreizung T -.75))
(add-sji (Augenreizung M -.75))
(add-sji (Augenreizung H -.75))

;Juckreiz
(add-sji (Juckreiz B -.75))
(add-sji (Juckreiz Q -.75))
(add-sji (Juckreiz K -.75))
|#


(sdp-fct (list (no-output (sdm isa symptom)) :creation-time -1e12 :reference-count 3740000))
(sdp-fct (list (no-output (sdm isa association)) :creation-time -1e12 :reference-count 3740000))
;(sdp-fct (list (no-output (sdm isa expectation)) :creation-time -1e12 :reference-count 3740000))


(sdp symptom1 :creation-time -1e12 :reference-count 3740000) ;full activation  -> should be retrieved in 3/4 of the cases
(sdp symptom2 :creation-time -1e12 :reference-count 3740000) ;full activation -> should be retrieved in 2/3 of the cases
(sdp symptom3 :creation-time -1e12 :reference-count 3740000) ;full activation

(sdp probe1 :creation-time -1e12 :reference-count 3450000)  ;1.931
(sdp probe2 :creation-time -1e12 :reference-count 3550000)  ;1.960
(sdp probe3 :creation-time -1e12 :reference-count 3740000) ;full activation
(sdp probe4 :creation-time -1e12 :reference-count 3740000)  ;full activation  ;2.012


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;1.1 diagnosis task (if a stimulus wider than 14 has been automatically stuffed in the visual-location buffer, move attention to this stimulus and set goal state to retrieve)

;new for expectation ->instead of one we need several to look at stimulus, depending on the expectation
;for the first symptom, no expectation is needed
(P look-at-stimulus-first
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     > width       10
  ?visual>
     state       free
  ?imaginal>
     state       free
     buffer      empty
==>
  +visual>
     ISA         move-attention
     screen-pos  =visual-location
  =goal>
     state       retrieve 
)

;after probe, no expectation is needed
(P look-at-stimulus-afterprobe
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     > width       10
  ?visual>
     state       free
  !eval!       (equalp *task* "probe")
==>
  +visual>
     ISA         move-attention
     screen-pos  =visual-location
  =goal>
     state       retrieve 
)

;for diagnosis, no expectation is needed
(P look-at-stimulus-diag  ;only necessary because after probe at pos 4, there is no expectation retrieved and then the other move-attention productions cannot fire
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     > width       10
  ?visual>
     state       free
  !eval!       (equalp *task* "diag")
==>
  +visual>
     ISA         move-attention
     screen-pos  =visual-location
  =goal>
     state       retrieve 
)

(spp look-at-stimulus-diag :u 1)   ;to avoid that another production fires at the question for the diagnosis

;the following productions are for the stimulus, when there is an expectation
;;;if the expectation is met, the stimulus is perceived
;;;if not, the expectation is changed
(P look-at-stimulus-symptom-expected1
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     > width       10
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          nil
  =retrieval>
     isa         expectation
     state       symptom
    !eval!      (setf *expect1* 'symptom)
==>
  =imaginal>
  +visual>
     ISA         move-attention
     screen-pos  =visual-location
  =goal>
     state       retrieve 
)

(P look-at-stimulus-symptom-expected2
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     > width       10
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          =s2
     s3          nil
  =retrieval>
     isa         expectation
     state       symptom
    !eval!      (setf *expect2* 'symptom)
==>
  =imaginal>
  +visual>
     ISA         move-attention
     screen-pos  =visual-location
  =goal>
     state       retrieve 
)

(P look-at-stimulus-symptom-expected3
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     > width       10
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          =s2
     s3          =s3
     s4          nil
  =retrieval>
     isa         expectation
     state       symptom
    !eval!      (setf *expect3* 'symptom)
==>
  =imaginal>
  +visual>
     ISA         move-attention
     screen-pos  =visual-location
  =goal>
     state       retrieve 
)

(P look-at-stimulus-symptom-expected4
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     > width       10
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          =s2
     s3          =s3
     s4          =s4
  =retrieval>
     isa         expectation
     state       symptom
    !eval!      (setf *expect4* 'symptom)
==>
  =imaginal>
  +visual>
     ISA         move-attention
     screen-pos  =visual-location
  =goal>
     state       retrieve 
)

(P look-at-stimulus-change-expectation
  =goal>
     ISA         diagnose
     state       change-expectation
  =visual-location>
     ISA         visual-location
     > width       10
  ?visual>
     state       free
  =retrieval>
     isa         expectation
     state       symptom
==>
  +visual>
     ISA         move-attention
     screen-pos  =visual-location
  =goal>
     state       retrieve 
)

(P look-at-stimulus-probe-expected1
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     > width       10
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          nil
  =retrieval>
     isa         expectation
     state       probe
    !eval!      (setf *expect1* 'probe)
==>
  =imaginal>
  =visual-location>
  =retrieval>
     state       symptom
  =goal>
     state       change-expectation
)

(P look-at-stimulus-probe-expected2
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     > width       10
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          =s2
     s3          nil
  =retrieval>
     isa         expectation
     state       probe
    !eval!      (setf *expect2* 'probe)
==>
  =imaginal>
  =visual-location>
  =retrieval>
     state       symptom
  =goal>
     state       change-expectation
)

(P look-at-stimulus-probe-expected3
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     > width       10
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          =s2
     s3          =s3
     s4          nil
  =retrieval>
     isa         expectation
     state       probe
    !eval!      (setf *expect3* 'probe)
==>
  =imaginal>
  =visual-location>
  =retrieval>
     state       symptom
  =goal>
     state       change-expectation
)

(P look-at-stimulus-probe-expected4
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     > width       10
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          =s2
     s3          =s3
     s4          =s4
  =retrieval>
     isa         expectation
     state       probe
    !eval!      (setf *expect4* 'probe)
==>
  =imaginal>
  =visual-location>
  =retrieval>
     state       symptom
  =goal>
     state       change-expectation
)

;01 probe task (if a stimulus smaller than 15 has been automatically stuffed in the visual-location buffer, move attention to this stimulus abd set goal state to retrieve-probe) 

(P look-at-probe-probe-expected1
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     < width       11
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          nil
  =retrieval>
     isa         expectation
     state       probe
    !eval!      (setf *expect1* 'probe)
==>
  =imaginal>
  +visual>
     ISA         move-attention
     screen-pos  =visual-location
  =goal>
     state       retrieve-probe 
)

(P look-at-probe-probe-expected2
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     < width       11
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          =s2
     s3          nil
  =retrieval>
     isa         expectation
     state       probe
    !eval!      (setf *expect2* 'probe)
==>
  =imaginal>
  +visual>
     ISA         move-attention
     screen-pos  =visual-location
  =goal>
     state       retrieve-probe 
)

(P look-at-probe-probe-expected3
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     < width       11
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          =s2
     s3          =s3
     s4          nil
  =retrieval>
     isa         expectation
     state       probe
    !eval!      (setf *expect3* 'probe)
==>
  =imaginal>
  +visual>
     ISA         move-attention
     screen-pos  =visual-location
  =goal>
     state       retrieve-probe 
)

(P look-at-probe-probe-expected4
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     < width       11
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          =s2
     s3          =s3
     s4          =s4
  =retrieval>
     isa         expectation
     state       probe
    !eval!      (setf *expect4* 'probe)
==>
  =imaginal>
  +visual>
     ISA         move-attention
     screen-pos  =visual-location
  =goal>
     state       retrieve-probe 
)

(P look-at-probe-change-expectation
  =goal>
     ISA         diagnose
     state       change-expectation
  =visual-location>
     ISA         visual-location
     < width       11
  ?visual>
     state       free
  =retrieval>
     isa         expectation
     state       probe
==>
  +visual>
     ISA         move-attention
     screen-pos  =visual-location
  =goal>
     state       retrieve-probe 
)



(P look-at-probe-symptom-expected1
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     < width       11
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          nil
  =retrieval>
     isa         expectation
     state       symptom
    !eval!      (setf *expect1* 'symptom)
==>
  =imaginal>
  =visual-location>
  =retrieval>
     state       probe
  =goal>
     state       change-expectation
)


(P look-at-probe-symptom-expected2
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     < width       11
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          =s2
     s3          nil
  =retrieval>
     isa         expectation
     state       symptom
    !eval!      (setf *expect2* 'symptom)
==>
  =imaginal>
  =visual-location>
  =retrieval>
     state       probe
  =goal>
     state       change-expectation
)

(P look-at-probe-symptom-expected3
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     < width       11
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          =s2
     s3          =s3
     s4          nil
  =retrieval>
     isa         expectation
     state       symptom
    !eval!      (setf *expect3* 'symptom)
==>
  =imaginal>
  =visual-location>
  =retrieval>
     state       probe
  =goal>
     state       change-expectation
)

(P look-at-probe-symptom-expected4
  =goal>
     ISA         diagnose
     state       start
  =visual-location>
     ISA         visual-location
     < width       11
  ?visual>
     state       free
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          =s2
     s3          =s3
     s4          =s4
  =retrieval>
     isa         expectation
     state       symptom
    !eval!      (setf *expect4* 'symptom)
==>
  =imaginal>
  =visual-location>
  =retrieval>
     state       probe
  =goal>
     state       change-expectation
)

;02 retrieve probe (if goal is to retrieve probe and a text is in the visual buffer, try to retrieve that text from the letter chunks in dm and set goal state to respond-to-probe)
(p retrieve-probe
 =goal>
    isa         diagnose
    state       retrieve-probe
  =visual>
     isa         text
     value       =probe
  ?retrieval>
     state       free
==>
;  =visual>              ;raus? - visual drfte damit leer sein
  +retrieval>
     ISA         association
     value       =probe
  =goal>
     state       respond-to-probe 
)

;03.1 respond yes (if goal state is respond-to-probe and a letter-chunk with value chemical has been retrieved, press y)
(p probe-yes
  =goal>
     ISA         diagnose
     state       respond-to-probe
  =retrieval>
     isa         association
     type        chemical
   ?manual>   
      state       free
==>
  =imaginal>
  =goal>
     state       start
   +manual>
      ISA         press-key
      key         y
   +visual>
    isa clear
)

;03.1 respond no  (if goal state is respond-to-probe and a letter-chunk with value distractor has been retrieved, press n)
(p probe-no
  =goal>
     ISA         diagnose
     state       respond-to-probe
  =retrieval>
     isa         association
     type        distractor
   ?manual>   
      state       free
==>
  =imaginal>
  =goal>
     state       start
   +manual>
      ISA         press-key
      key         n
   +visual>
     isa clear
)


;2.1 retrieve chunk of current stimulus, if stimulus is not 'diagnosis'
(P retrieve-chunk-of-current-stimulus
  =goal>
     ISA         diagnose
     state       retrieve
  =visual>
     isa         text
     value       =text
     - value     "diagnosis"
  ?retrieval>
     state       free
==>
 ; =visual>                   ;raus? - visual drfte damit leer sein
  +retrieval>
     ISA         symptom
     value       =text
  =goal>
     state       encode 
)

;2.3 if you don't know the presented stimulus (can't retrieve a chunk of its meaning), set goal to start again
(p dummy
  =goal>
     ISA         diagnose
     state       encode
  ?retrieval>
     state       error
==>
   =goal>
      state       start
)

;3.1 ->back to 1
;encode first symptom (if you retrieved a symptom-chunk, and the imaginal buffer is empty, put the chunk in the first slot of the imaginal buffer)
(P encode-current-stimulus-first-time
  =goal>
     ISA         diagnose
     state       encode
  =retrieval>
     isa         symptom
     value       =text
;  =visual>                    ;raus?
 ;    isa         text
 ;    value       =text
  ?imaginal>
     state       free
     buffer      empty
==>
  +imaginal>
     ISA         previoussymptom
     s1          =retrieval      ;e.g.: (cough isa symptom value "cough")
  +retrieval>
     ISA         expectation
     position    one
  =goal>
     state       start
  +visual>
   isa clear
)


;3.2 -> back to 1
;encode second symptom (if you retrieved a symptom-chunk, and something is in the first slot of the imaginal buffer, push that into the second slot of the imaginal and put the retrieved chunk in the first slot of the imaginal buffer)
(P encode-current-stimulus2
  =goal>
     ISA         diagnose
     state       encode
  =retrieval>
     isa         symptom
     value       =text
;  =visual>
 ;   isa         text
  ;  value       =text
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          nil
==>
  =imaginal>
     s1          =retrieval       ;e.g.: (shortbreath isa symptom value "shortbreath")
     s2          =s1              ;e.g.: (itching isa symptom value "itching")
  +retrieval>
     ISA         expectation
     position    two
  =goal>
     state       start
  +visual>
    isa clear
)

;3.3 -> back to 1
;encode third symptom (if you retrieved a symptom-chunk, and something is in the first and second slot of the imaginal buffer, push that into the second / third slot of the imaginal and put the retrieved chunk in the first slot of the imaginal buffer)
(P encode-current-stimulus3
  =goal>
     ISA         diagnose
     state       encode
  =retrieval>
     isa         symptom
     value       =text
;     - value     "diagnosis"   
 ; =visual>
 ;   isa         text
 ;   value       =text
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          =s2
     s3          nil
==>
  =imaginal>
     s1          =retrieval       ;e.g.: (shortbreath isa symptom value "shortbreath")
     s2          =s1              ;e.g.: (itching isa symptom value "itching")
     s3          =s2              ;e.g.: (cough isa symptom value "cough")
  +retrieval>
     ISA         expectation
     position    three
  =goal>
     state       start
  +visual>
    isa clear
)

;3.4 -> back to 1
;encode fourth symptom (if you retrieved a symptom-chunk, and something is in the first, second and third slot of the imaginal buffer, push that into the second / third /fourth slot of the imaginal and put the retrieved chunk in the first slot of the imaginal buffer)
(P encode-current-stimulus4
  =goal>
     ISA         diagnose
     state       encode
  =retrieval>
     isa         symptom
     value       =text
;     - value     "diagnosis"
;  =visual>
;     isa         text
;     value       =text
  =imaginal>
     isa         previoussymptom
     s1          =s1
     s2          =s2
     s3          =s3
==>
  =imaginal>
     s1          =retrieval       ;e.g.: (headache isa symptom value "headache")
     s2          =s1              ;e.g.: (shortbreath isa symptom value "shortbreath")
     s3          =s2              ;e.g.: (itching isa symptom value "itching")
     s4          =s3              ;e.g.: (cough isa symptom value "cough")
  +retrieval>
     ISA         expectation
     position    four
  =goal>
     state       start
  +visual>
    isa clear
)


;2.2 retrieve diagnosis  (if stimulus in the visual buffer is 'diagnosis', retrieve association-chunk with the highest activation)
(p diagnosis
   =goal>
     ISA         diagnose
     state       retrieve
  =visual>
     isa         text
     value       "diagnosis"
  ?retrieval>
     state       free
==>
 ; =visual>                          ;raus?
  +retrieval>
     ISA         association
  =goal>
     state       respond
)


;4 -> back to one
;respond with diagnosis (if you retrieved an association chunk, type its name) and harvest the imaginal buffer
(p respond
   =goal>
      ISA         diagnose
      state       respond
   =retrieval>
      isa         association
      value       =diagnosis
 ;  =imaginal>
 ;     isa         previoussymptom
   ?manual>
      state       free
==>
   -imaginal>   ;JPB08090 no theoretical difference, except that this rule can still fire
                ;         when there's no imaginal
   =goal>
      state       start
   +manual>
      ISA         press-key
      key         =diagnosis
   +visual>
     isa clear ;just to make sure this doesn't cause a problem
)



(goal-focus goal)

)